//*****************************************************************************
//*****************************************************************************
//  FILENAME:   feature.c
//  Version: 1.0, 2006/10/24 
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Handling of FEATURE requests
//
//  NOTE: The Toradex Definition is based on 32-Bytes Feature reports. 
// 		  Therefore, the file usb_descr.asm needs to be modified after each
//        "Generate Application":
//		  1) USB_INTERFACE_0_FEATURE_RPT_DATA: BLK  32
//		  2) USB_HID_RPT_1_FEATURE_RPT_SIZE:   EQU  32
//		  3) Add a Label at the beginning of the string descriptor containing
//           the USB serial number:
//				EXPORT _USB_Serial:
//				_USB_Serial:
//		  4) Put all the USB string descriptors into 
//			 ROM area "StringDescriptors"
//		  5) in usb_drv.asm, add the following label and export it:
//			 _USB_TransferType  (identical to USB_TranferType, but with 
//			 					 leading underscore)
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
// This is special version because the parameter PAR16_OFFSET is not set to the
// initial value at the time the serial number is set (see ProcessFeatureReports() "case 20").
// The parameter PAR16_OFFSET can be set previous (calibrartion is done and 
// then the standart test program is used).
#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "feature.h"
#include "digitalio.h"
#include "Nonvolatile.h"
#include "string.h"
#include "usb.h"
#include "parameters.h"
#include "sensor.h"

//----------------------------------------
// The following status Variable is modified by the Cypress USB firmware
//----------------------------------------
extern BYTE USB_TransferType;
#define USB_TRANS_STATE_IDLE 0x00

//----------------------------------------
// definition for USB serial string
// 	when replacing the Serial, the length of the new serial MUST equal
// 	the current string size!
//----------------------------------------
#define ANYLENGTH 255
typedef struct
{
	BYTE	bLength;          // descriptor size in Bytes, including 
	BYTE	bDescriptorType;  // must be 3. DO NOT CHANGE
	char	UnicodeText[ANYLENGTH]; // Unicode text with little endian byte order
} tStringDescriptor;

extern const tStringDescriptor USB_Serial;

//----------------------------------------
// - return data
#define FEAT_RETURN				0xFF

//-----------------------------------------------------------------------------
// compare a ROM memory range against a RAM memory range
// returns TRUE if they are equal
//-----------------------------------------------------------------------------
BOOL cmemcmp (const char *rom, char *ram, BYTE length)
{
	while (length>0)
	{
		length--;
		if ( rom[length] != ram[length] )
			return FALSE;
	}
	return TRUE;
}
//-----------------------------------------------------------------------------
// Initialize Feature Reporting
//-----------------------------------------------------------------------------
void Feature_Start(void)
{
	FEATURE_RPT.IN.ReturnCode = 0xFF; // to mark that there is no command pending
	Parameters_Start();
}

//-----------------------------------------------------------------------------
// check if feature report is pending
// if yes:
//	1) process request
//  2) prepare return data if appropriate
//  3) set FSM_FeatureRpt to allow further processing
//-----------------------------------------------------------------------------
void ProcessFeatureReports (void)
{
	BYTE	Result;
	BYTE	i;
	char	txt[21];
	WORD	w;
	DWORD	dw;
	BYTE	NumberOfReturnedBytes = 0;

	// if return data has been overwritten by a command and SetFeature has finished
	if ( (USB_TransferType == USB_TRANS_STATE_IDLE) && (FEATURE_RPT.IN.ReturnCode != 0xFF) )
	{
		switch (FEATURE_RPT.OUT.Target)
		{
			// ---------- target is RAM or Flash ==> standard processing
			case eRAM:		
			case eFlash:
				if (FEATURE_RPT.OUT.Get_nSet)
				{	// ----- GET
					switch (FEATURE_RPT.OUT.ParamSize)
					{
						case   1:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_8)
							{
								Get_Param_8(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, (BYTE*)FEATURE_RPT.IN.Param);
								NumberOfReturnedBytes = 1;
							}
							break;

						case  2:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_16)
							{
								Get_Param_16(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, &w);
								FEATURE_RPT.IN.Param[0]= w & 0xFF;
								FEATURE_RPT.IN.Param[1]= w / 0x100;
								NumberOfReturnedBytes = 2;
							}
							break;

						case  4:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_32)
							{
								Get_Param_32(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, &dw);
								FEATURE_RPT.IN.Param[0]=  w        & 0xFF;
								FEATURE_RPT.IN.Param[1]= (w / 0x100) & 0xFF;
								FEATURE_RPT.IN.Param[2]= (w / 0x10000) & 0xFF;
								FEATURE_RPT.IN.Param[3]= (w / 0x1000000) & 0xFF;
								NumberOfReturnedBytes = 4;
							}
							break;

						case 21:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_168)
							{
								Get_Param_168(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, FEATURE_RPT.IN.Param);
								NumberOfReturnedBytes = 21;
							}
							break;
					}
				}
				else	
				{	// ----- SET
					switch (FEATURE_RPT.OUT.ParamSize)
					{
						case   1:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_8)
								Set_Param_8(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, FEATURE_RPT.OUT.Param[0]);
							break;
						
						case  2:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_16)
								Set_Param_16(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, FEATURE_RPT.OUT.Param[0]+256*FEATURE_RPT.OUT.Param[1]);
							break;
						
						case  4:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_32)
								Set_Param_32(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, 
								             FEATURE_RPT.OUT.Param[0]+0x100*FEATURE_RPT.OUT.Param[1]+0x10000*FEATURE_RPT.OUT.Param[2]+0x1000000*FEATURE_RPT.OUT.Param[3]);
							break;
						
						case 21:
							if (FEATURE_RPT.OUT.Index_LSB < NUM_PARAM_168)
								Set_Param_168(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Target, FEATURE_RPT.OUT.Param);
							break;
						
						case 20: // only for "set USB serial"
							if (( cmemcmp ("USBSN", &FEATURE_RPT.OUT.Param[0], 5) ) && ( USB_Serial.bLength == FEATURE_RPT.OUT.Param[5] ))
								// modify USB serial number
								Result = Flash_Modify ((const BYTE*)&USB_Serial, &FEATURE_RPT.OUT.Param[5], USB_Serial.bLength);
								
							// reset all parameters to factory defaults, even if the security bytes above are incorrect
							for (i=0; i < NUM_PARAM_8; i++)
								Set_Param_8( i, NONVOLATILE, FactoryDefault_Param_8[i] );
							for (i=0; i < NUM_PARAM_16; i++)
							{
								if (PAR16_OFFSET !=i)	// offset is not set back to factory setting
								{
									Set_Param_16( i, NONVOLATILE, FactoryDefault_Param_16[i] );
								}									
							}								
							for (i=0; i < NUM_PARAM_32; i++)
								Set_Param_32( i, NONVOLATILE, FactoryDefault_Param_32[i] );
							for (i=0; i < NUM_PARAM_168; i++)
							{
								cstrncpy(txt, FactoryDefault_Param_168[i], 20); // copy from Flash to RAM (necessary due to Harvard architecture)
								Set_Param_168( i, NONVOLATILE, txt );
							}

							break;
					}
					NumberOfReturnedBytes = 0;
					Sensor_Specific_Parameter(&NumberOfReturnedBytes);  // Allows for special treatment at the end of Parameter Set / Get command.
				}	
				break;	
			
			// ---------- special Treatment for Parameters outside of Flash or RAM
			case eController:	
				switch (FEATURE_RPT.OUT.Index_LSB)
				{
					case 0:
						if ( (FEATURE_RPT.OUT.Index_LSB ==0) && (FEATURE_RPT.OUT.ParamSize == 3) )
							if ( FEATURE_RPT.OUT.Get_nSet )
								FEATURE_RPT.IN.Param[0] = DigitalIn_Get();
							else
								DigitalOut_Set (FEATURE_RPT.OUT.Param[0], FEATURE_RPT.OUT.Param[1], FEATURE_RPT.OUT.Param[2]);
						NumberOfReturnedBytes = 0;
						break;	
						
					default:
						NumberOfReturnedBytes = Sensor_Specific_Feature();
						break;
				}
			
			case eSensor:
			case eAction:
				NumberOfReturnedBytes = Sensor_Specific_Feature();
				break;

		}	
		
		// clear unused part of report to 0x00
		memset(&FEATURE_RPT.IN.Param[NumberOfReturnedBytes], 0xAA, 31-NumberOfReturnedBytes);
		FEATURE_RPT.IN.ReturnCode = 0xFF;
	}
}